/*
 * Gxl2TouchGraph.java
 *
 * Copyright (C) 2003 Miika Nurminen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
import java.io.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import java.awt.image.BufferedImage;



class NodeAction {
  public void doAction(Element src,Element target) {
  }
}

/**
 * Conversion program from GXL to TouchGraph 1.20
 * Converted to standard filter 19.9.2003
 *
 * @version 0.1 - 2003-8-6
 * @author  minurmin
 */
public class Gxl2TouchGraph {
  // Global value so it can be ref'd by the tree-adapter
  Document document;
  Document outDoc;
  String workDir = "";
  
  private String getAbsoluteFileName(String fname) {
    return StringLib.catOnce(workDir,File.separator)+fname;
  }
  
  /**
   * Creates a DOM representation of an XML document given in filename.
   *
   * @param filename input file
   */
  public void fromFile( InputStream is,String workDir) throws SAXException,ParserConfigurationException,IOException {
    this.workDir = workDir;
    DocumentBuilderFactory factory =
    DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    document = builder.parse( is/*new File(filename)*/ );
  }
  
  
  
  // add content action class here...
  private void addSubNode(Element src, Element target, String name, NodeAction action)  {
    Element e = outDoc.createElement(name);
    action.doAction(src,e);
    target.appendChild(e);
  }
  
  private void transformNodes(Element root) {
    NodeList nl = document.getElementsByTagName("node");
    for (int i=0; i<nl.getLength(); i++) {
      if (!(nl.item(i) instanceof Element)) continue;
      Element e = (Element)(nl.item(i));
      
      Element n = outDoc.createElement("NODE");
      n.setAttribute("nodeID", e.getAttribute("id"));
      root.appendChild(n);
      
      final Map nodeAttrMap = GxlLib.getNodeAttributes(e);
      
      addSubNode(e,n,"NODE_LOCATION", new NodeAction() {
        public void doAction(Element src,Element target) {
          MapStringParser p = new MapStringParser();
          Map m = p.parse(src.getAttribute("style"));
          target.setAttribute("x", StringLib.removePostfix(m.get("left").toString(),"px"));
          target.setAttribute("y", StringLib.removePostfix(m.get("top").toString(),"px"));
          target.setAttribute("visible","true");
        }
      });
      
      addSubNode(e,n,"NODE_LABEL",new NodeAction() {
        public void doAction(Element src,Element target) {
          //          Map m = GxlLib.getNodeAttributes(src);
          target.setAttribute("label",nodeAttrMap.get("label").toString());
          if (nodeAttrMap.get("concept").equals("true")) {
            target.setAttribute("shape","1");
            target.setAttribute("backColor","6060F0");
            target.setAttribute("textColor","FFFFFF");
            target.setAttribute("fontSize","17");
          }
          else {
            target.setAttribute("shape","2");
            target.setAttribute("backColor","FFFFFF");
            target.setAttribute("textColor","000000");
            target.setAttribute("fontSize","12");
          }
        }
      });
      
      addSubNode(e,n,"NODE_URL",new NodeAction() {
        public void doAction(Element src,Element target) {
          target.setAttribute("url","");
          target.setAttribute("urlIsLocal","false");
          target.setAttribute("urlIsXML","false");

          Iterator i = nodeAttrMap.keySet().iterator();
          while (i.hasNext()) {
            String s = i.next().toString();
            if (s.indexOf("resource")<0) continue;
            
            MapStringParser p = new MapStringParser();
            Map m = p.parse(nodeAttrMap.get(s).toString());
            String imgname = StringLib.removePrefix(m.get("relativeURL").toString(),"file:/./");

            File f = new File(imgname);
            String tempname = f.getName();
            if ((StringLib.testEndStr(imgname,".url")) && (tempname.indexOf("ext_")==0)) {
                try {
                  BufferedReader r = new BufferedReader(new InputStreamReader(new FileInputStream(getAbsoluteFileName(imgname))));
                  target.setAttribute("url",r.readLine());
                }
                catch (IOException e) {
                  System.err.println(e.getMessage());
                  e.printStackTrace();
                }
            }
          }
        }
      });
      
      addSubNode(e,n,"NODE_HINT",new NodeAction() {
        
        public void doAction(Element src,Element target) {
          Iterator i = nodeAttrMap.keySet().iterator();
          StringBuffer hintValue = new StringBuffer(100);
          Attr a = outDoc.createAttribute("hint");
          int minWidth = 250;
          while (i.hasNext()) {
            String s = i.next().toString();
            if (s.indexOf("resource")<0) continue;
            
            MapStringParser p = new MapStringParser();
            Map m = p.parse(nodeAttrMap.get(s).toString());
            String imgname = StringLib.removePrefix(m.get("relativeURL").toString(),"file:/./");

            File f = new File(imgname);
            String tempname = f.getName();
            
            // this is a direct node link
            if ((StringLib.testEndStr(imgname,".url")) && (tempname.indexOf("ext_")==0)) {
              hintValue.append("Double-click to navigate.");
            }
            else {
//            System.err.println(tempname+" "+tempname.indexOf("in_"));
            if ( (tempname.indexOf("in_")==0) && (
              StringLib.testEndStr(imgname,(".jpg")) ||
              StringLib.testEndStr(imgname,(".png")) ||
              StringLib.testEndStr(imgname,(".gif")) ||
              StringLib.testEndStr(imgname,(".jpeg")) )
            ) {
              hintValue.append("<img title=\""+m.get("desc")+"\" src=\""+imgname+"\">");
              try {
                BufferedImage b = ImgLib.readImage(getAbsoluteFileName(imgname));
                if ((b.getWidth()+20)>minWidth) minWidth=b.getWidth()+20;
              }
              catch (IOException e) { // if image cannot be read, it is simply skipped.
                System.err.println(e.getMessage());
                e.printStackTrace();
              }
            }
            else {
              // normal links, concept map handling. to do: if more prefixes/postfixes occur, generalize!
              String targetFrame = "_blank";
              if (StringLib.testEndStr(imgname,".cmap")) {
                // removing path, changing extension                         // file.separator account only separator of current system
                imgname = "map.php?map="+imgname.substring(Math.max(imgname.lastIndexOf('/'),imgname.lastIndexOf('\\'))+1,imgname.length()-5)/*+".html"*/;
                targetFrame="_self"; // concept maps are opened on same frame
              }
              else if (StringLib.testEndStr(imgname,".url")) {
                try {
                  BufferedReader r = new BufferedReader(new InputStreamReader(new FileInputStream(getAbsoluteFileName(imgname))));
                  imgname = r.readLine();
                }
                catch (IOException e) {
                  System.err.println(e.getMessage());
                  e.printStackTrace();
                }
              }
              hintValue.append("<a target=\""+targetFrame+"\" href=\""+imgname+"\">"+m.get("desc").toString()+"</a>");
            }
            hintValue.append("<br>");
          }
          
          }
          target.setAttribute("hint",hintValue.toString());
          
          target.setAttribute("width", Integer.toString(minWidth)/*"300"*/); 
          target.setAttribute("height","-1");
          target.setAttribute("isHTML","true");
        }
      });
      
    }
  }
  
  private void transformEdges(Element root) {
    NodeList nl = document.getElementsByTagName("edge");
    for (int i=0; i<nl.getLength(); i++) {
      if (!(nl.item(i) instanceof Element)) continue;
      Element e = (Element)(nl.item(i));
      
      Element n = outDoc.createElement("EDGE");
      
      /* model attributes */
      n.setAttribute("fromID", e.getAttribute("from"));
      n.setAttribute("toID", e.getAttribute("to"));

      final Map nodeAttrMap = GxlLib.getNodeAttributes(e);
      n.setAttribute("label",  nodeAttrMap.get("label").toString());
      
      /* layout attributes */
      n.setAttribute("type","1");
      n.setAttribute("length","40");
      n.setAttribute("visible","true");
      n.setAttribute("color","A0A0A0");
      
      root.appendChild(n);
    }
  }
  
  private void insertParameter(Element root,String key,String value) {
    Element e = outDoc.createElement("PARAM");
    e.setAttribute("name",key);
    e.setAttribute("value",value);
    root.appendChild(e);
  }
  
  private void transformParameters(Element root) {
    insertParameter(root,"offsetX","300");
    insertParameter(root,"offsetY","300");
    insertParameter(root,"rotateSB","0");
    insertParameter(root,"zoomSB","-5");
    insertParameter(root,"localitySB","2");
    insertParameter(root,"backSelectColor","#C0C0FF");
  }
  
  public void doTransform() throws ParserConfigurationException {
    DocumentBuilderFactory factory =
    DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();

    try {
    outDoc = builder.parse( this.getClass().getResourceAsStream("input.xml") );
    } catch(org.xml.sax.SAXException e) {
     
    }
    catch (java.io.IOException e) {
      System.out.println("Error reading file input.xml!");
    }
  
    transformNodes((Element)(outDoc.getElementsByTagName("NODESET").item(0)));
    transformEdges((Element)(outDoc.getElementsByTagName("EDGESET").item(0)));
    transformParameters((Element)(outDoc.getElementsByTagName("PARAMETERS").item(0)));
  }
  
  /**
   * Transforms DOM representation to stream using XHTML 1.0 Transitional doctype.
   *
   * @param os Writer output stream writer
   */
  public void toStream( OutputStream os /*Writer os, String outname*/) throws TransformerException, FileNotFoundException, IOException {
    this.outDoc.normalize();
    DOMSource domSource = new DOMSource(this.outDoc);
    StreamResult streamResult = new StreamResult(os);
    TransformerFactory tf = TransformerFactory.newInstance();
    // XSL transformation is required to set correct doctype.
    Transformer serializer = tf.newTransformer(new StreamSource( this.getClass().getResourceAsStream("transformoutput.xsl") /*"transformoutput.xsl"*/));
    serializer.setOutputProperty(OutputKeys.ENCODING,"ISO-8859-1");
    serializer.setOutputProperty(OutputKeys.INDENT,"yes");
    serializer.transform(domSource, streamResult);
  }
  
  /**
   * Main program.
   */
  public static void main(String[] args) {
    FileArgumentScheme m = new FileArgumentScheme(args);
    String workDir = m.determineFilename("-dir");
    if (workDir==null) workDir = System.getProperty("user.dir");
    try {
      if (m.helpWanted()) {
        System.out.println("\nConverts GXL to KeyGraph (adapted TouchGraph) XML file.");
        System.out.println("Operates on standard input / output or given filenames.");
        System.out.println("Relative working directory can be defined using -dir switch.");
        System.out.println("Requires libraries jgraph.jar and jgraphpad.jar\n");
        System.out.println("Usage examples:");
        System.out.println("  cat input.xml | java Gxl2TouchGraph > output.xml");
        System.out.println("  java -dir ../mydir/ -i input.xml -o output.xml\n");
        return;
      }
      m.openStreams();
      Gxl2TouchGraph t = new Gxl2TouchGraph();
      t.fromFile(m.getInput(),workDir);
      t.doTransform();
    
      
      t.toStream(m.getOutput()); // takes outfilename without extension
    }
    catch (Exception e) {
    /* Potential Exception types (omitted for clarity):
        -SAXException
        -ParserConfigurationException
        -TransformerException
        -IOException */
      
      ErrorLib.errorReport(e);
    }
    try {
      m.getInput().close();
      m.getOutput().close();
    }
    catch (IOException e) {
      ErrorLib.errorReport(e);
    }

  }
  
}