/*
 * Xcm2Gxl.java
 *
 * Copyright (C) 2003 Miika Nurminen
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/* This library includes modified code from JGraphPad 3.0.
 */

import java.awt.Font;
import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.*;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.jgraph.*;
import org.jgraph.graph.*;
import org.jgraph.pad.GPUserObject;
import org.jgraph.pad.GPConverter;
import org.jgraph.pad.GPGraph;

// to do: versio ilman ikkonointia? (mahdollisesti toinen luokka ikkunointia varten)

/** 
 * Adapted from org.jgraph.pad.GPConverter
 */
class MyGPConverter {
  // Create a GXL-representation for the specified cells.
  static transient Hashtable hash;

  /**
   * Modification (25.7.2003): doctype. / Miika Nurminen
   */
  public static String toGXL(GPGraph graph, Object[] cells) {
    hash = new Hashtable();
    String gxl = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"+
//    "<!DOCTYPE gxl SYSTEM \"http://www.gupro.de/GXL/gxl-1.0.dtd\">\n"+
    "<gxl xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n<graph id=\"1\" edgeids=\"true\" edgemode=\"directed\">";
                                          // graphwiz requires graph attributes. current id is not very useful, though... /mn, 6.1.2003
    
    // Create external keys for nodes
    for (int i = 0; i < cells.length; i++)
      if (graph.isVertex(cells[i])) {
        
        GPUserObject target = (GPUserObject)(((DefaultGraphCell)(cells[i])).getUserObject());
        Object id = target.getProperty("id");
        
        hash.put(cells[i], id/*new Integer(hash.size())*/);
      }
    
    // Convert Nodes
    Iterator it = hash.keySet().iterator();
    while (it.hasNext()) {
      Object node = it.next();
      gxl += vertexGXL(graph, hash.get(node), node);
    }
    
    // Convert Edges
    int edges = 0;
    for (int i = 0; i < cells.length; i++)
      if (graph.isEdge(cells[i])) {
        GPUserObject target = (GPUserObject)(((DefaultEdge)(cells[i])).getUserObject());
        gxl += edgeGXL(graph, 
        target.getProperty("id").toString(),
        ((DefaultEdge)(cells[i])).getUserObject().toString(), cells[i]);
      }
    // Close main tags
    gxl += "\n</graph></gxl>";
    return gxl;
  }
  
  
  private static void addAttribute(StringBuffer str,String key,String value) {
    str.append("\n\t\t<attr name=\"");
    str.append(key);
    str.append("\">\n\t\t\t<string>");
    str.append(value);
    str.append("</string>\n\t\t</attr>");
  }
  
  /**
   * Modification (25.7.2003): added layout information. / Miika Nurminen
   */
  public static String vertexGXL(GPGraph graph, Object id, Object vertex) {
    Rectangle bounds = graph.getCellBounds(vertex);    
    String stylestr = "style=\"left="+bounds.x+"px;top="+bounds.y+"px;width="+bounds.width+"px;height="+bounds.height+"px;\"";
    StringBuffer nodeStr = new StringBuffer(100);
    nodeStr.append("\n\t<node id=\""+id.toString()+ "\" "+stylestr+">");

    GPUserObject target = (GPUserObject)(((DefaultGraphCell)(vertex)).getUserObject());
    Map m = target.getProperties();
    Iterator i = m.keySet().iterator();
    while (i.hasNext()) {
      String key = i.next().toString();
      if (!key.equals("id")) {
        String value = m.get(key).toString();
        if (key.equals("value")) key = "label";
        addAttribute(nodeStr,key, value);
      }
    }
    
    nodeStr.append("\n\t</node>");
    return nodeStr.toString();
  }
  
  public static String edgeGXL(GPGraph graph, String id,String label, Object edge) {
    GraphModel model = graph.getModel();
    String from = "";
    if (model.getSource(edge) != null) {
      Object source = hash.get(model.getParent(model.getSource(edge)));
      if (source != null)
        from = /*"node" +*/ source.toString();
    }
    String to = "";
    if (model.getTarget(edge) != null) {
      Object target = hash.get(model.getParent(model.getTarget(edge)));
      if (target != null)
        to = /*"node" +*/ target.toString();
    }
    if (from != null && to != null) {
      StringBuffer result = new StringBuffer(100);
      result.append("\n\t<edge id=\""+ id + "\" from=\"");
      result.append(from);
      result.append("\" to=\"");
      result.append(to);
      result.append("\">");
      addAttribute(result, "label", label);
      result.append("</edge>");
      return result.toString();
    } else
      return "";
  }
}

class MyGPUserObject extends org.jgraph.pad.GPUserObject {
  
  public MyGPUserObject(String label) {
    super(label);
  }
        // added method 3.9.2003 / mn
	public void showPublicPropertyDialog(final GPGraph graph, final Object cell) {
          showPropertyDialog(graph,cell);
        }

}

/**
 * Test class based on JGraph tutorial.
 * 
 * Converted to standard filter 19.9.2003
 *
 * @version 0.1 / 24.7.2003
 * @author  minurmin
 */
public class Xcm2Gxl {
  
 
  class MyTestGraph extends GPGraph {

    
    // Construct the Graph using the Model as its Data Source
    public MyTestGraph(GraphModel model) {
      super(model);
      // Use a Custom Marquee Handler
      setMarqueeHandler(new MyMarqueeHandler());
    }
    
    
/*	public Object[] getAll() {
		return getDescendants(getRoots());
	}*/
  }
  
  class MyMarqueeHandler extends BasicMarqueeHandler {
    // Display PopupMenu or Remember Start Location and First Port
    
    // Override to Gain Control (for PopupMenu and ConnectMode)
    
    public boolean isForceMarqueeEvent(MouseEvent e) {
      // If Right Mouse Button we want to Display the PopupMenu
      if (SwingUtilities.isRightMouseButton(e))
        // Return Immediately
        return true;
      // Else Call Superclass
      return super.isForceMarqueeEvent(e);
    }
    
    public void mousePressed(final MouseEvent e) {
      // If Right Mouse Button
      if (SwingUtilities.isRightMouseButton(e)) {
        Point loc = graph.fromScreen(e.getPoint());
        // Find Cell in Model Coordinates
        Object cell = graph.getFirstCellForLocation(loc.x, loc.y);
        
        if (cell!=null) {
          final Object userObject = ((DefaultGraphCell) cell).getUserObject();
          if (userObject instanceof MyGPUserObject) {
            ((MyGPUserObject) userObject).showPublicPropertyDialog(graph, cell);
          }
        }
        
        // Else if in ConnectMode and Remembered Port is Valid
      } else
        // Call Superclass
        super.mousePressed(e);
    }
    
    
  }
  
  
  
  Document doc = null;
  Map ids = null;  // Hashtable for the ID lookup (ID to StorableObject, ie. all objects in IHMC model)
  GPGraph graph;
  
  
  /**
   * Parses filename out of IHMC resourcelink / resourceref attribute.
   */
  private String parseResourceRef(String input) {
    StringTokenizer st = new StringTokenizer(input,";");
    while (st.hasMoreTokens()) {
      String current = st.nextToken();
      if (current.indexOf("relativeURL")>=0) {
        return current;
      }
    }
    return "";
  }
  
  private String getChildAttribute(Node n, String attrname) {
    NodeList nl = n.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      if (nl.item(i) instanceof Element) {
        Element e = (Element)(nl.item(i));
        if (e.getAttribute("name").equals(attrname))
          return e.getAttribute("encoding");
      }
    }
    return null;
  }
  
  /**
   * Extracts node position information
   */
  void fetchNodeViewProperties(Node gnode, Map map) {
    String s = getChildAttribute(gnode,"_selectedGraphicalRep");
    Element e = (Element)(ids.get(s)); // getelementbyid does not work - no DTD
    //      Node n = doc.getElementById(s);
    int x = Integer.parseInt(getChildAttribute(e,"_x"));
    int y = Integer.parseInt(getChildAttribute(e,"_y"));
    //      gnode_attrs = JGraph.createBounds(x,y, Color.black);
    Color c = Color.blue;
    GraphConstants.setBounds(map, new Rectangle(x, y, 80, 30));
    GraphConstants.setAutoSize(map, true);
    GraphConstants.setBorder(map, BorderFactory.createRaisedBevelBorder());
    GraphConstants.setBackground(map, c.darker());
    GraphConstants.setForeground(map, Color.white);
    GraphConstants.setFont(map, GraphConstants.defaultFont.deriveFont(Font.BOLD, 12));
    GraphConstants.setOpaque(map, true);
    
  }
  
  /**
   * Extracts edge label position information
   */
  void fetchEdgeViewProperties(Node gnode, Map gnode_attrs) {
    String s = getChildAttribute(gnode,"_selectedGraphicalRep");
    Element e = (Element)(ids.get(s)); // getelementbyid does not work - no DTD
    //      Node n = doc.getElementById(s);
    int x = Integer.parseInt(getChildAttribute(e,"_x"));
    int y = Integer.parseInt(getChildAttribute(e,"_y"));
    GraphConstants.setBounds(gnode_attrs, new Rectangle(x,y, 5, 5));
    GraphConstants.setAutoSize(gnode_attrs, true);
  }
  
 
  /**
   * Returns ids of concepts where current linkphrase is connected.
   * @param inboundId secondnode of the source concept
   */
  private ArrayList getOutboundConnections(Map ids,String inboundId) {
    ArrayList result = new ArrayList(2);

    Iterator i = ids.keySet().iterator();
    while (i.hasNext()) {
      String id = (String)(i.next());
      Element e = (Element)(ids.get(id));
      if (e.getAttribute("subclass").equals("nlk.base.Connection")) { // we found a connection!
        if (getChildAttribute(e,"_firstNode").equals(inboundId)) {
          result.add(getChildAttribute(e,"_secondNode"));
        }
      }
    }  
    return result;
  }
  
  /**
   * Creates the cells defined in the GXL file, and constructs the arguments for 
   * the insert method, namely a list of cells, a connection set, and a map, 
   * from cells to attributes. To establish the connections, the vertices in the 
   * file are given unique IDs, which are referenced from the edge's source and 
   * target. (The ids map is used to look-up the vertices.)
   */
  public void performTransform(InputStream f, OutputStream fout, GPGraph graph) throws Exception {
    GraphModel model=graph.getModel();
    
    // Create a DocumentBuilderFactory
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    // Create a DocumentBuilder
    DocumentBuilder db = dbf.newDocumentBuilder();
    // Parse the input file to get a Document object
    doc = db.parse(f);
    
    // List for the new Cells
    List newCells = new ArrayList();
    
    // ConnectionSet for the Insert method
    ConnectionSet cs = new ConnectionSet();
    
    // Hashtable for Attributes (Vertex to Map)
    Hashtable attributes = new Hashtable();
    
   
    ids = new Hashtable();
    Map vertexIds = new Hashtable();
    
    NodeList nl;
    Element e;
    // clearing presentation-specific (and partially invalid) *-attributes
    nl = doc.getElementsByTagName("storableObject");
    
    for (int i=0; i<nl.getLength(); i++) {
      if (nl.item(i) instanceof Element) {
        e = (Element)(nl.item(i));
        String id = e.getAttribute("id");
        ids.put(id,e);
      }
    }
    
    
    Iterator i = ids.keySet().iterator();
    while (i.hasNext()) {
      MyGPUserObject target = null;
      String id = (String)(i.next());
      e = (Element)(ids.get(id));
      if (e.getAttribute("subclass").equals("nlk.base.Concept")) { // ok, we have a concept. add it to our model
        String label = getLabel(e);
        label = label.replaceAll("\\^linebreak\\$","");//^$
        target = new MyGPUserObject(label);
        DefaultGraphCell vertex = new DefaultGraphCell(target);
        target.putProperty("id", id);
        target.putProperty("concept","true");

        // Add One Floating Port
        vertex.add(new DefaultPort());
        // Add ID, Vertex pair to Hashtable
        // Add Default Attributes
        Map node_attrs = createDefaultAttributes();
        fetchNodeViewProperties(e, node_attrs);
        attributes.put(vertex, node_attrs);
        
        // Add Vertex to new Cells
        newCells.add(vertex);
        vertexIds.put(id,vertex);
        
      }
      else if (e.getAttribute("subclass").equals("nlk.base.LinkingPhrase")) { // link node (actually edge label)
/*        String label = getChildAttribute(e,"_phrase");  

        target = new MyGPUserObject(label);
        DefaultGraphCell vertex = new DefaultGraphCell(target);
        target.putProperty("id", id);
        target.putProperty("concept","false");

        vertex.add(new DefaultPort());
        Map node_attrs = createDefaultAttributes();
        fetchEdgeViewProperties(e, node_attrs);
        attributes.put(vertex, node_attrs);
        
        // Add Vertex to new Cells
        newCells.add(vertex);
        vertexIds.put(id,vertex);*/
      }
    }
    
    i = ids.keySet().iterator();
    while (i.hasNext()) {
      String id = (String)(i.next());
      e = (Element)(ids.get(id));
      if (e.getAttribute("subclass").equals("nlk.base.Connection")) { // we found a connection!
        String first = getChildAttribute(e,"_firstNode");
        String second = getChildAttribute(e,"_secondNode");
        Element e2 = (Element)ids.get(second);
        if (e2.getAttribute("subclass").equals("nlk.base.Concept")) continue;
        String label = getChildAttribute(e2, "_phrase");
        ArrayList a = getOutboundConnections(ids, second);
        for (int j=0; j<a.size(); j++) {
          MyGPUserObject target = new MyGPUserObject(label);
          target.putProperty("id", id+"_"+j);
          DefaultEdge edge = new DefaultEdge(target);
          
          cs.connect(edge,((DefaultGraphCell)(vertexIds.get(first))).getChildAt(0),((DefaultGraphCell)(vertexIds.get(a.get(j)))).getChildAt(0));
          newCells.add(edge);
        }
       
/*        DefaultEdge edge = new DefaultEdge(id);
        String first = getChildAttribute(e,"_firstNode");
        String second = getChildAttribute(e,"_secondNode");
        
        cs.connect(edge,((DefaultGraphCell)(vertexIds.get(first))).getChildAt(0),((DefaultGraphCell)(vertexIds.get(second))).getChildAt(0));
        newCells.add(edge);*/
      }
    }
  
    
    i = ids.keySet().iterator();
    while (i.hasNext()) {
      String id = (String)(i.next());
      e = (Element)(ids.get(id));
      if (e.getAttribute("subclass").equals("nlk.base.ResourceGroup")) { // we found a resourcegroup
        String concept = getChildAttribute(e,"_node");
        String resources = getChildAttribute(e,"_resourceLinks");
        
        DefaultGraphCell currentVertex = (DefaultGraphCell)(vertexIds.get(concept));
        MyGPUserObject target = (MyGPUserObject)(currentVertex.getUserObject());
        Element resvec = (Element)(ids.get(resources));

        NodeList links = resvec.getChildNodes();
        for (int j=0; j<links.getLength(); j++) {
          if (links.item(j) instanceof Element) {
            Element linkref = (Element)(links.item(j));
            if (linkref.getAttribute("subclass").equals("nlk.base.ResourceLink")) {
              Element currentLink = (Element)(ids.get(linkref.getAttribute("encoding")));
              String propvalue = parseResourceRef(getChildAttribute(currentLink,"_resourceRef"));
              String resName = getChildAttribute(currentLink,"_resourceName");
              String resDesc = getChildAttribute(currentLink,"_resourceDescription");
              String counter = Integer.toString(target.getProperties().size()-2);
              target.putProperty("resource_"+counter, propvalue+";name="+resName+";desc="+resDesc);
            }
          }
        }
      }
    }

    // Insert the cells (View stores attributes)
    model.insert(newCells.toArray(), attributes, cs, null, null);
    fout.write(MyGPConverter.toGXL(graph,graph.getAll()).getBytes());
  }
  
  
  /**
   * Returns an attributeMap for the specified position and color.
   */
  public Map createDefaultAttributes() {
    // Create an AttributeMap
    Map map = GraphConstants.createMap();
    // Set a Black Line Border (the Border-Attribute must be Null!)
    GraphConstants.setBorderColor(map, Color.black);
    // Return the Map
    return map;
  }
  
  // Fetch Cell Label from Node
  protected String getLabel(Element e) {
    NodeList nl = e.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      if (!(nl.item(i) instanceof Element)) continue;
      Element n = (Element)(nl.item(i));
      if (n.getAttribute("name").equals("_phrase"))
        return n.getAttribute("encoding");
    }
    return "";
  }
  
  /** Creates a new instance of JGraphTest */
  public Xcm2Gxl() {
  }
  
  public void run(InputStream in, PrintStream out,boolean window) {
    graph = new MyTestGraph(new DefaultGraphModel());
    
    try {
      performTransform(in,out, graph);
      if (!window) return;

      JFrame frame = new JFrame(JGraph.VERSION);
      frame.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
          System.exit(0);
        }
      });
      
      // Resize (Otherwise not Visible)
      graph.setSize(graph.getPreferredSize());
      graph.setDragEnabled(true);
      JScrollPane j = new JScrollPane(graph);
      frame.getContentPane().add(j);
      frame.pack();
      frame.show();
    }
    catch(Exception e) {
      System.out.println(e.toString());
      e.printStackTrace();
    }
  }
  
  
  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) {
    boolean keepWindow=(StringLib.arrayIndexOf(args,"-nowindow",true)<0);
    FileArgumentScheme m = new FileArgumentScheme(args);
    try {
      if (m.helpWanted()) {
        System.out.println("\nConverts IHMC Concept Map tool 3.0 XCM file to GXL.");
        System.out.println("Operates on standard input / output or given filenames.");
        System.out.println("If -nowindow switch is given, does not open any windows.");
        System.out.println("Requires libraries jgraph.jar and jgraphpad.jar\n");
        System.out.println("Usage examples:");
        System.out.println("  cat input.xml | java -cp jgraph.jar:jgraphpad.jar Xcm2Gxl > output.xml");
        System.out.println("  java -cp jgraph.jar:jgraphpad.jar Xcm2Gxl -nowindow -i input.xml -o output.xml\n");
        return;
      }
      m.openStreams();
      Xcm2Gxl application = new Xcm2Gxl();
      application.run(m.getInput(),m.getOutput(),keepWindow);
    }
    catch (IOException e) {
      ErrorLib.errorReport(e);
    }
    try {
      m.getInput().close();
      m.getOutput().close();
    }
    catch (IOException e) {
      ErrorLib.errorReport(e);
    }
  }
}